require 'uu/os/form/input'

module UU
  module OS
    module Form

      # Component for entering text.
      class Text < UU::OS::Form::Input
        
        # Type of input. Can be one of "text", "password" or "number".
        attr_accessor :type
        
        # Max count of characters typed into input.
        attr_accessor :max_length
        
        # Input restrictions.
        attr_accessor :restriction
        
        # The format of a number using Unicode CLDR notation. Supported formatting
        # characters are '#', '0', ',' (thousand separator) and '.' (decimal separator).
        # Note that ',' and '.' are not the actual separators. Instead, they'll be
        # replaced according to the user's language settings.
        #
        # @example
        #   # standard format (displays "12,345.6" in English, "12 345,6" in Czech, ...)
        #   comp.number_format = "#,##0.################"
        #
        #   # integers only
        #   comp.number_format = "#,##0"
        #
        #   # integers, without thousand separators
        #   comp.number_format = "#"
        #
        #   # display exactly 2 decimal digits
        #   comp.number_format = "#,##0.00"
        #
        #   # display at least 2 decimal digits
        #   comp.number_format = "#,##0.00##############"
        #
        # @see http://cldr.unicode.org/translation/number-patterns
        attr_accessor :number_format

        # The color to use for positive numbers and zero. Applies only when type="NUMBER"
        # and read_only=true. Allowed values are 6-digit hexadecimal color codes prefixed
        # with #, e.g. "#ff0000" and logical names of colors from UU colormap, e.g.
        # "red" or "priorityHigh".
        attr_accessor :positive_number_color

        # The color to use for negative numbers. Applies only when type="NUMBER"
        # and read_only=true. Allowed values are 6-digit hexadecimal color codes prefixed
        # with #, e.g. "#ff0000" and logical names of colors from UU colormap, e.g.
        # "red" or "priorityHigh".
        attr_accessor :negative_number_color

        # The styling of the component. This component allows styling using following
        # CSS attributes:
        # * text-align - alignment of the component text ("left", "right", "center").
        attr_accessor :style

        # The value as it is displayed to the user (after formatting). For example using type="NUMBER"
        # and number_format="#,##0" and value="-1234", the displayed value for a Czech user
        # would be "-1 234".
        attr_reader :displayed_value

        def type
          attributes[:type]
        end
        
        def type=(type)
          attributes[:type] = type
        end

        def max_length
          attributes[:maxLength]
        end

        def max_length=(max_length)
          attributes[:maxLength] = max_length
        end

        def restriction
          attributes[:restriction]
        end

        def restriction=(restriction)
          attributes[:restriction] = restriction
        end
                
        def number_format
          attributes[:numberFormat]
        end

        def number_format=(value)
          attributes[:numberFormat] = value
        end

        def positive_number_color
          attributes[:positiveNumberColor]
        end

        def positive_number_color=(value)
          attributes[:positiveNumberColor] = value
        end

        def negative_number_color
          attributes[:negativeNumberColor]
        end

        def negative_number_color=(value)
          attributes[:negativeNumberColor] = value
        end

        def style
          attributes[:style]
        end

        def style=(value)
          attributes[:style] = value
        end

        def displayed_value
          attributes[:displayedValue]
        end

      end
    end
  end
end