require 'json'

module UU
  module OS
    module GVC

      # GVC message (used by components)
      class Message

        # Raw message attributes.
        # @private
        attr_reader :raw_attrs

        # ID of component which message belongs to.
        attr_reader :related_to

        # Message code.
        attr_accessor :code

        # Message severity. Allowed values {UU::OS::GVC::Message::MessageSeverity}.
        attr_accessor :severity

        # Message text.
        attr_accessor :text

        # Creates new instance of message.
        #
        # @param [String, Hash] data Initialization data in form of JSON string or Hash (optional)
        def initialize(data = nil)
          if data.nil?
            @raw_attrs = {}
            return
          end
          if data.kind_of?String
            data = JSON.parse(data, :symbolize_names => true)
          end
          if data.kind_of?Hash
            @raw_attrs = data
          elsif data.kind_of?Message
            if data.raw_attrs
              @raw_attrs = data.raw_attrs
            else
              @raw_attrs = {}
            end
          else
            raise ArgumentError.new("Message data must be JSON String or Hash, but was #{data.class}.")
          end
          if !@raw_attrs[:relatedTo]
            @raw_attrs[:relatedTo] = []
          end
        end

        def raw_attrs
          # Disallow manipulating with raw attributes
          @raw_attrs.clone if @raw_attrs
        end

        def related_to
          @raw_attrs[:relatedTo]
        end

        def code
          @raw_attrs[:code]
        end

        def code=(val)
          @raw_attrs[:code] = val
        end

        def severity
          @raw_attrs[:severity]
        end

        def severity=(val)
          @raw_attrs[:severity] = val
        end

        def text
          @raw_attrs[:text]
        end

        def text=(val)
          @raw_attrs[:text] = val
        end

        # @private
        def to_s
          "[#{severity ? severity : 'NA'}] #{code ? code : 'NA'}: #{text}"
        end

        # Enumeration of message severity levels.
        class MessageSeverity

          # Info message.
          INFO = 'information'

          # Warning message.
          WARNING = 'warning'

          # Error message.
          ERROR = 'error'

        end

      end

    end
  end
end
